@@ -22,14 +22,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """Trollmoves client."""
-
+import argparse
 import logging
 import os
 import socket
-import sys
 import time
 from collections import deque
-from configparser import RawConfigParser
+from configparser import ConfigParser
 from threading import Lock, Thread, Event
 import hashlib
 from urllib.parse import urlparse, urlunparse
@@ -41,11 +40,13 @@ from zmq import LINGER, POLLIN, REQ, Poller
 import bz2
 from posttroll import get_context
 from posttroll.message import Message, MessageError
-from posttroll.publisher import NoisyPublisher
+from posttroll.publisher import create_publisher_from_dict_config
 from posttroll.subscriber import Subscriber
 from trollsift.parser import compose
 
 from trollmoves import heartbeat_monitor
+from trollmoves.logging import add_logging_options_to_parser
+from trollmoves.move_it_base import MoveItBase
 from trollmoves.utils import get_local_ips
 from trollmoves.utils import gen_dict_extract, translate_dict
 from trollmoves.movers import CTimer
@@ -71,17 +72,24 @@ BUNZIP_BLOCK_SIZE = 1024
 LISTENER_CHECK_INTERVAL = 1
 
 
+def is_localhost(host):
+    """Check if host is localhost."""
+    return socket.gethostbyname(host) in get_local_ips()
+
+
 def read_config(filename):
     """Read the config file called *filename*."""
-    cp_ = RawConfigParser()
-    cp_.read(filename)
+    cp_ = ConfigParser(interpolation=None)
+    with open(filename) as config_file:
+        cp_.read_file(config_file)
 
     res = {}
 
     for section in cp_.sections():
         res[section] = dict(cp_.items(section))
         _set_config_defaults(res[section])
-        _parse_boolean_config_items(res[section])
+        _parse_boolean_config_items(res[section], cp_[section])
+        _parse_nameservers(res[section], cp_[section])
         if not _check_provider_config(res, section):
             continue
         if not _check_destination(res, section):
@@ -104,19 +112,24 @@ def _set_config_defaults(conf):
     conf.setdefault("create_target_directory", True)
 
 
-FALSY = ["", "False", "false", "0", "off"]
-TRUTHY = ["True", "true", "on", "1"]
+def _parse_boolean_config_items(conf, raw_conf):
+    for key in ["delete", "heartbeat", "create_target_directory"]:
+        try:
+            val = raw_conf.getboolean(key)
+        except ValueError:
+            continue
+        if val is not None:
+            conf[key] = val
 
 
-def _parse_boolean_config_items(conf):
-    if conf["delete"] in FALSY:
-        conf["delete"] = False
-    if conf["delete"] in TRUTHY:
-        conf["delete"] = True
-    if conf["heartbeat"] in FALSY:
-        conf["heartbeat"] = False
-    if conf["create_target_directory"] in FALSY:
-        conf["create_target_directory"] = False
+def _parse_nameservers(conf, raw_conf):
+    try:
+        val = raw_conf.getboolean("nameservers")
+    except ValueError:
+        val = conf["nameservers"]
+    if isinstance(val, str):
+        val = val.split()
+    conf["nameservers"] = val
 
 
 def _check_provider_config(conf, section):
@@ -167,7 +180,7 @@ class Listener(Thread):
 
     def __init__(self, address, topics, *args, die_event=None, **kwargs):
         """Init Listener object."""
-        super(Listener, self).__init__()
+        super().__init__()
 
         self.topics = topics
         self.subscriber = None
@@ -278,7 +291,7 @@ def _handle_push_message(msg):
         # the transfers are not finished on primary
         # client and are not cleared
         LOGGER.debug("Primary client published 'push'")
-        add_to_ongoing(msg)
+        add_to_ongoing_transfers(msg)
         return True
     return False
 
@@ -295,7 +308,7 @@ def _handle_ack_message(msg):
 def _handle_message_from_another_client(msg):
     if msg.type == "file" and "request_address" not in msg.data:
         LOGGER.debug("Ignoring 'file' message from primary client.")
-        add_to_ongoing(msg)
+        add_to_ongoing_transfers(msg)
         _ = add_to_file_cache(msg)
         _ = clean_ongoing_transfer(get_msg_uid(msg))
         return True
@@ -396,7 +409,7 @@ def resend_if_local(msg, publisher):
     """Resend the message provided all uris point to local files."""
     for uri in gen_dict_extract(msg.data, 'uri'):
         urlobj = urlparse(uri)
-        if not publisher or not socket.gethostbyname(urlobj.netloc) in get_local_ips():
+        if not publisher or not is_localhost(urlobj.netloc):
             return
 
     LOGGER.debug('Sending: %s', str(msg))
@@ -477,27 +490,32 @@ def unpack_and_create_local_message(msg, local_dir, **kwargs):
 def make_uris(msg, destination, login=None):
     """Create local URIs for the received files."""
     duri = urlparse(destination)
-    scheme = duri.scheme or 'ssh'
-    dest_hostname = duri.hostname or socket.gethostname()
-    if scheme not in ('s3') and socket.gethostbyname(dest_hostname) in get_local_ips():
-        scheme_, host_ = "ssh", dest_hostname  # local file
-    else:
-        scheme_, host_ = scheme, dest_hostname  # remote file
+    scheme = duri.scheme
+    netloc = duri.netloc
+    if scheme != "s3" and empty_or_localhost(duri.hostname):
+        scheme = ""
+        netloc = ""
+    elif netloc:
         if login:
             # Add (only) user to uri.
-            host_ = login.split(":")[0] + "@" + host_
+            netloc = login.split(":")[0] + "@" + netloc
 
     def uri_callback(var):
         uid = var['uid']
         path = os.path.join(duri.path, uid)
-        var['uri'] = urlunparse((scheme_, host_, path, "", "", ""))
+        var['uri'] = urlunparse((scheme, netloc, path, "", "", ""))
         return var
     msg.data = translate_dict(msg.data, ('uri', 'uid'), uri_callback)
     return msg
 
 
+def empty_or_localhost(hostname):
+    """Check that hostname is either empty or referring to localhost."""
+    return ((not hostname) or (hostname and is_localhost(hostname)))
+
+
 def replace_mda(msg, kwargs):
-    """Replace messate metadata with items in kwargs dict."""
+    """Replace message metadata with items in kwargs dict."""
     for key in msg.data:
         if key in kwargs:
             try:
@@ -575,23 +593,23 @@ def add_request_push_timer(timeout, msg, *args, **kwargs):
     LOGGER.debug("Added timer for UID %s.", huid)
 
 
-def add_to_ongoing(msg):
+def add_to_ongoing_transfers(msg):
     """Add message to ongoing transfers.
 
-    Return True if similar message was already received, False otherwise.
+    Return None if similar message was already received, otherwise the hashed uid of the message.
     """
-    huid = get_msg_uid(msg)
+    hashed_uid = get_msg_uid(msg)
     with hot_spare_timer_lock:
-        timer = ongoing_hot_spare_timers.pop(huid, None)
+        timer = ongoing_hot_spare_timers.pop(hashed_uid, None)
         if timer is not None:
             timer.cancel()
-            LOGGER.debug("Cleared timer for UID %s.", huid)
+            LOGGER.debug("Cleared timer for UID %s.", hashed_uid)
     with ongoing_transfers_lock:
-        if huid in ongoing_transfers:
-            ongoing_transfers[huid].append(msg)
+        if hashed_uid in ongoing_transfers:
+            ongoing_transfers[hashed_uid].append(msg)
             return None
-        ongoing_transfers[huid] = [msg]
-        return huid
+        ongoing_transfers[hashed_uid] = [msg]
+        return hashed_uid
 
 
 def add_to_file_cache(msg):
@@ -605,31 +623,31 @@ def add_to_file_cache(msg):
 
 def request_push(msg_in, destination, login=None, publisher=None, **kwargs):
     """Request a push for data."""
-    huid = add_to_ongoing(msg_in)
-    if huid is None:
+    hashed_uid = add_to_ongoing_transfers(msg_in)
+    if hashed_uid is None:
         return
 
     if already_received(msg_in):
         timeout = float(kwargs["req_timeout"])
         send_ack(msg_in, timeout)
-        _ = clean_ongoing_transfer(huid)
+        _ = clean_ongoing_transfer(hashed_uid)
         return
 
-    _request_files(huid, destination, login, publisher, **kwargs)
+    _request_files(hashed_uid, destination, login, publisher, **kwargs)
 
 
-def _request_files(huid, destination, login, publisher, **kwargs):
-    for msg in iterate_messages(huid):
+def _request_files(hashed_uid, destination, login, publisher, **kwargs):
+    for msg in iterate_messages(hashed_uid):
         _destination = _compose_destination(destination, msg)
 
-        req, fake_req = create_push_req_message(msg, _destination, login)
-        LOGGER.info("Requesting: %s", str(fake_req))
+        req, no_credentials_req = create_push_req_message(msg, _destination, login)
+        LOGGER.info("Requesting: %s", str(no_credentials_req))
         if kwargs.get('create_target_directory', True):
             local_dir = create_local_dir(_destination, kwargs.get('ftp_root', '/'))
         else:
             local_dir = None
 
-        publisher.send(str(fake_req))
+        publisher.send(str(no_credentials_req))
 
         response, hostname = send_request(msg, req, float(kwargs["transfer_req_timeout"]))
 
@@ -637,6 +655,7 @@ def _request_files(huid, destination, login, publisher, **kwargs):
             LOGGER.debug("Server done sending file")
             add_to_file_cache(msg)
             _send_ack_message(msg, publisher)
+
             try:
                 lmsg = unpack_and_create_local_message(response, local_dir, **kwargs)
                 lmsg = _update_local_message(lmsg, _destination, login, response, **kwargs)
@@ -646,7 +665,7 @@ def _request_files(huid, destination, login, publisher, **kwargs):
 
             LOGGER.debug("publishing %s", str(lmsg))
             publisher.send(str(lmsg))
-            terminate_transfers(huid, float(kwargs["req_timeout"]))
+            terminate_transfers(hashed_uid, float(kwargs["req_timeout"]))
             break
         else:
             LOGGER.error("Failed to get valid response from server %s: %s",
@@ -654,7 +673,7 @@ def _request_files(huid, destination, login, publisher, **kwargs):
     else:
         LOGGER.warning('Could not get a working source for requesting %s',
                        str(msg))
-        terminate_transfers(huid, float(kwargs["req_timeout"]))
+        terminate_transfers(hashed_uid, float(kwargs["req_timeout"]))
 
 
 def _compose_destination(destination, msg):
@@ -697,11 +716,11 @@ class Chain(Thread):
 
     def __init__(self, name, config):
         """Init a chain object."""
-        super(Chain, self).__init__()
+        super().__init__()
         self._config = config
         self._name = name
-        self._np = None
         self.publisher = None
+        self._pub_starter = None
         self.listeners = {}
         self.listener_died_event = Event()
         self.running = True
@@ -709,18 +728,16 @@ class Chain(Thread):
 
     def setup_publisher(self):
         """Initialize publisher."""
-        if self._np is None:
-            try:
+        if self.publisher is None:
+            with suppress(KeyError, NameError):
                 nameservers = self._config["nameservers"]
-                if nameservers:
-                    nameservers = nameservers.split()
-                self._np = NoisyPublisher(
-                    "move_it_" + self._name,
-                    port=self._config["publish_port"],
-                    nameservers=nameservers)
-                self.publisher = self._np.start()
-            except (KeyError, NameError):
-                pass
+                pub_settings = {
+                    "name": "move_it_" + self._name,
+                    "port": self._config["publish_port"],
+                    "nameservers": nameservers,
+                }
+                self._pub_starter = create_publisher_from_dict_config(pub_settings)
+                self.publisher = self._pub_starter.start()
 
     def setup_listeners(self, keep_providers=None):
         """Set up the listeners."""
@@ -854,9 +871,10 @@ class Chain(Thread):
         self.reset_listeners()
 
     def _stop_publisher(self):
-        if self._np:
-            self._np.stop()
-            self._np = None
+        if self.publisher:
+            self._pub_starter.stop()
+            self._pub_starter = None
+            self.publisher = None
 
     def restart(self):
         """Restart the chain, return a new running instance."""
@@ -901,7 +919,7 @@ def reload_config(filename, chains):
     LOGGER.debug("Reloaded config from %s", filename)
 
 
-class PushRequester(object):
+class PushRequester:
     """Base requester class."""
 
     request_retries = 3
@@ -921,7 +939,6 @@ class PushRequester(object):
     def connect(self):
         """Connect to the server."""
         self._socket = get_context().socket(REQ)
-        # self._socket.setsockopt(zmq.RCVTIMEO, 500)  # milliseconds
         self._socket.connect(self._reqaddress)
         self._poller.register(self._socket, POLLIN)
 
@@ -994,12 +1011,42 @@ class PushRequester(object):
         return rep
 
 
-def terminate(chains):
-    """Terminate client chains."""
-    for chain in chains.values():
-        chain.stop()
-    LOGGER.info("Shutting down.")
-    print("Thank you for using pytroll/move_it_client."
-          " See you soon on pytroll.org!")
-    time.sleep(1)
-    sys.exit(0)
+def parse_args(args=None):
+    """Parse commandline arguments."""
+    parser = argparse.ArgumentParser()
+    parser.add_argument("config_file",
+                        help="The configuration file to run on.")
+    add_logging_options_to_parser(parser, legacy=True)
+    return parser.parse_args(args)
+
+
+class MoveItClient(MoveItBase):
+    """Trollmoves client class."""
+
+    def __init__(self, cmd_args):
+        """Initialize client."""
+        self.name = "move_it_client"
+        super().__init__(cmd_args)
+
+    def reload_cfg_file(self, filename, *args, **kwargs):
+        """Reload configuration file."""
+        reload_config(filename, self.chains, *args, **kwargs)
+
+    def signal_reload_cfg_file(self, *args):
+        """Handle reload signal."""
+        reload_config(self.cmd_args.config_file, self.chains,
+                      publisher=self.publisher)
+
+    def _run(self):
+        for chain_name in self.chains:
+            if not self.chains[chain_name].is_alive():
+                self.chains[chain_name] = self.chains[chain_name].restart()
+            self.chains[chain_name].publisher.heartbeat(30)
+
+    def terminate(self):
+        """Terminate client chains."""
+        for chain in self.chains.values():
+            chain.stop()
+        LOGGER.info("Shutting down.")
+        print("Thank you for using pytroll/move_it_client."
+              " See you soon on pytroll.org!")
