@@ -26,73 +26,53 @@
 import logging
 import logging.handlers
 import os
+import signal
+import time
+from threading import Lock
+from abc import ABC, abstractmethod
+from contextlib import suppress
 
 import pyinotify
 from posttroll.publisher import Publisher
 
 LOGGER = logging.getLogger("move_it_base")
-LOG_FORMAT = "[%(asctime)s %(levelname)-8s %(name)s] %(message)s"
 
 
-class MoveItBase(object):
+class MoveItBase(ABC):
     """Base class for Trollmoves."""
 
-    def __init__(self, cmd_args, chain_type, publisher=None):
+    def __init__(self, cmd_args, publisher=None):
         """Initialize the class."""
         self.cmd_args = cmd_args
-        self.chain_type = chain_type
         self.running = False
         self.notifier = None
         self.watchman = None
         self.publisher = publisher
-        self._np = None
         self.chains = {}
-        setup_logging(cmd_args, chain_type)
         LOGGER.info("Starting up.")
-        self.setup_watchers(cmd_args)
-
-    def reload_cfg_file(self, filename, *args, **kwargs):
-        """Reload configuration file."""
-        if self.chain_type == "client":
-            from trollmoves.client import reload_config
-            reload_config(filename, self.chains, *args, **kwargs)
-        else:
-            # Also Mirror uses the reload_config from the Server
-            from trollmoves.server import reload_config
-            reload_config(filename, self.chains, *args, publisher=self.publisher,
-                          use_watchdog=self.cmd_args.watchdog,
-                          disable_backlog=self.cmd_args.disable_backlog)
-
-    def signal_reload_cfg_file(self, *args):
-        """Handle reload signal."""
-        del args
-        if self.chain_type == "client":
-            from trollmoves.client import reload_config
-            reload_config(self.cmd_args.config_file, self.chains,
-                          publisher=self.publisher)
-        else:
-            from trollmoves.server import reload_config
-            reload_config(self.cmd_args.config_file, self.chains,
-                          publisher=self.publisher,
-                          use_watchdog=self.cmd_args.watchdog,
-                          disable_backlog=self.cmd_args.disable_backlog)
+        self.setup_watchers()
+        self.run_lock = Lock()
 
     def chains_stop(self, *args):
         """Stop all transfer chains."""
         del args
-        if self.chain_type == "client":
-            from trollmoves.client import terminate
-        else:
-            from trollmoves.server import terminate
-        self.running = False
-        self.notifier.stop()
-        try:
-            self._np.stop()
-        except AttributeError:
-            pass
-        terminate(self.chains)
+        with suppress(RuntimeError):
+            self.run_lock.acquire(timeout=1)
 
-    def setup_watchers(self, cmd_args):
+        self.running = False
+        try:
+            self.notifier.stop()
+        except RuntimeError as err:
+            LOGGER.warning("Could not stop notifier: %s", err)
+        with suppress(AttributeError):
+            self.publisher.stop()
+        self.terminate()
+
+    @abstractmethod
+    def terminate(self):
+        """Terminate the chains and threads."""
+
+    def setup_watchers(self):
         """Set up watcher for the configuration file."""
         mask = (pyinotify.IN_CLOSE_WRITE |
                 pyinotify.IN_MOVED_TO |
@@ -104,35 +84,30 @@ class MoveItBase(object):
                                      tmask=mask,
                                      cmd_filename=self.cmd_args.config_file)
         self.notifier = pyinotify.ThreadedNotifier(self.watchman, event_handler)
-        self.watchman.add_watch(os.path.dirname(cmd_args.config_file), mask)
+        self.watchman.add_watch(os.path.dirname(self.cmd_args.config_file), mask)
 
+    def run(self):
+        """Start the transfer chains."""
+        try:
+            signal.signal(signal.SIGTERM, self.chains_stop)
+            signal.signal(signal.SIGHUP, self.signal_reload_cfg_file)
+        except ValueError:
+            LOGGER.warning("Signals could not be set up.")
+        self.notifier.start()
+        self.running = True
+        while self.running:
+            time.sleep(1)
+            shutting_down = not self.run_lock.acquire(blocking=False)
+            if shutting_down:
+                break
+            try:
+                self._run()
+            finally:
+                self.run_lock.release()
 
-def setup_logging(cmd_args, chain_type):
-    """Set up logging."""
-    global LOGGER
-    LOGGER = logging.getLogger('')
-    if cmd_args.verbose:
-        LOGGER.setLevel(logging.DEBUG)
-
-    if cmd_args.log:
-        fh_ = logging.handlers.TimedRotatingFileHandler(
-            os.path.join(cmd_args.log),
-            "midnight",
-            backupCount=7)
-    else:
-        fh_ = logging.StreamHandler()
-
-    formatter = logging.Formatter(LOG_FORMAT)
-    fh_.setFormatter(formatter)
-
-    LOGGER.addHandler(fh_)
-    logger_name = "move_it_server"
-    if chain_type == "client":
-        logger_name = "move_it_client"
-    elif chain_type == "mirror":
-        logger_name = "move_it_mirror"
-    LOGGER = logging.getLogger(logger_name)
-    pyinotify.log.handlers = [fh_]
+    @abstractmethod
+    def _run(self):
+        raise NotImplementedError
 
 
 def create_publisher(port, publisher_name):
