@@ -24,18 +24,17 @@
 """Movers for the move_it scripts."""
 
 import logging
+import netrc
 import os
 import shutil
 import sys
 import time
 import traceback
+import socket
+from ftplib import FTP, all_errors, error_perm
 from threading import Event, Lock, Thread, current_thread
 from urllib.parse import urlparse
-import netrc
 
-from ftplib import FTP, all_errors, error_perm
-from paramiko import SSHClient, SSHException, AutoAddPolicy
-from scp import SCPClient
 try:
     from s3fs import S3FileSystem
 except ImportError:
@@ -90,6 +89,7 @@ class Mover(object):
     """Base mover object. Doesn't do anything as it has to be subclassed."""
 
     def __init__(self, origin, destination, attrs=None):
+        """Initialize the Mover."""
         try:
             self.destination = urlparse(destination)
         except AttributeError:
@@ -184,13 +184,13 @@ class CTimer(Thread):
 
     """
 
-    def __init__(self, interval, function, args=(), kwargs={}):
+    def __init__(self, interval, function, args=(), kwargs=None):
         """Initialize the timer."""
         Thread.__init__(self)
         self.interval = interval
         self.function = function
         self.args = args
-        self.kwargs = kwargs
+        self.kwargs = kwargs or {}
         self.finished = Event()
 
     def cancel(self):
@@ -212,7 +212,7 @@ class FtpMover(Mover):
     active_connection_lock = Lock()
 
     def _get_netrc_authentication(self):
-        """Get login authentications from netrc file if available"""
+        """Get login authentications from netrc file if available."""
         try:
             secrets = netrc.netrc()
         except (netrc.NetrcParseError, FileNotFoundError) as e__:
@@ -297,27 +297,31 @@ class ScpMover(Mover):
 
     def open_connection(self):
         """Open a connection."""
+        from paramiko import SSHClient, SSHException
+
         retries = 3
         ssh_key_filename = self.attrs.get("ssh_key_filename", None)
+        timeout = self.attrs.get("ssh_connection_timeout", None)
         while retries > 0:
             retries -= 1
             try:
                 ssh_connection = SSHClient()
-                ssh_connection.set_missing_host_key_policy(AutoAddPolicy())
                 ssh_connection.load_system_host_keys()
                 ssh_connection.connect(self.destination.hostname,
                                        username=self._dest_username,
                                        port=self.destination.port or 22,
-                                       key_filename=ssh_key_filename)
+                                       key_filename=ssh_key_filename,
+                                       timeout=timeout)
                 LOGGER.debug("Successfully connected to %s:%s as %s",
                              self.destination.hostname,
                              self.destination.port or 22,
                              self._dest_username)
             except SSHException as sshe:
-                LOGGER.error("Failed to init SSHClient: %s", str(sshe))
+                LOGGER.exception("Failed to init SSHClient: %s", str(sshe))
+            except socket.timeout as sto:
+                LOGGER.exception("SSH connection timed out: %s", str(sto))
             except Exception as err:
-                LOGGER.error("Unknown exception at init SSHClient: %s",
-                             str(err))
+                LOGGER.exception("Unknown exception at init SSHClient: %s", str(err))
             else:
                 return ssh_connection
 
@@ -353,6 +357,8 @@ class ScpMover(Mover):
 
     def copy(self):
         """Upload the file."""
+        from scp import SCPClient
+
         ssh_connection = self.get_connection(self.destination.hostname,
                                              self.destination.port or 22,
                                              self._dest_username)
@@ -387,69 +393,25 @@ class SftpMover(Mover):
     """Move files over sftp."""
 
     def move(self):
-        """Push it !"""
+        """Push the file."""
         self.copy()
         os.remove(self.origin)
 
-    def _agent_auth(self, transport):
-        """Attempt to authenticate to the given transport using any of the private
-        keys available from an SSH agent ... or from a local private RSA key file
-        (assumes no pass phrase).
+    def copy(self):
+        """Copy files.
 
-        PFE: http://code.activestate.com/recipes/576810-copy-files-over-ssh-using-paramiko/
+        Uses high level paramiko functions.
         """
         import paramiko
-
-        agent = paramiko.Agent()
-
-        private_key_file = self.attrs.get("ssh_private_key_file", None)
-        if private_key_file:
-            private_key_file = os.path.expanduser(private_key_file)
-            LOGGER.info("Loading keys from local file %s", private_key_file)
-            agent_keys = (paramiko.RSAKey.from_private_key_file(private_key_file),)
-        else:
-            LOGGER.info("Loading keys from SSH agent")
-            agent_keys = agent.get_keys()
-        if len(agent_keys) == 0:
-            raise IOError("No available keys")
-
-        for key in agent_keys:
-            LOGGER.debug('Trying ssh key %s',
-                         key.get_fingerprint().encode('hex'))
-            try:
-                transport.auth_publickey(self._dest_username, key)
-                LOGGER.debug('... ssh key success!')
-                return
-            except paramiko.SSHException:
-                continue
-
-        # We found no valid key
-        raise IOError("RSA key auth failed!")
-
-    def copy(self):
-        """Upload the file."""
-        import paramiko
-
-        transport = paramiko.Transport((self.destination.hostname,
-                                        self.destination.port or 22))
-        transport.start_client()
-
-        self._agent_auth(transport)
-
-        if not transport.is_authenticated():
-            raise IOError("RSA key auth failed!")
-
-        sftp = transport.open_session()
-        sftp = paramiko.SFTPClient.from_transport(transport)
-        # sftp.get_channel().settimeout(300)
-
-        try:
-            sftp.mkdir(os.path.dirname(self.destination.path))
-        except IOError:
-            # Assuming remote directory exist
-            pass
-        sftp.put(self.origin, self.destination.path)
-        transport.close()
+        with paramiko.SSHClient() as ssh:
+            ssh.load_system_host_keys()
+            ssh.connect(self.destination.hostname,
+                        port=self.destination.port or 22,
+                        username=self._dest_username,
+                        allow_agent=True,
+                        key_filename=self.attrs.get("ssh_private_key_file"))
+            with ssh.open_sftp() as sftp:
+                sftp.put(self.origin, self.destination.path)
 
 
 class S3Mover(Mover):
